
/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.jms.wireformat;

import java.io.DataInputStream;
import java.io.DataOutputStream;

import org.jboss.jms.client.delegate.ClientConsumerDelegate;
import org.jboss.jms.delegate.SessionEndpoint;
import org.jboss.jms.destination.JBossDestination;

/**
 * 
 * A SessionCreateConsumerDelegateRequest
 *
 * @author <a href="mailto:tim.fox@jboss.com">Tim Fox</a>
 * @version <tt>$Revision: 2684 $</tt>
 *
 * $Id: SessionCreateConsumerDelegateRequest.java 2684 2007-05-15 07:31:30Z timfox $
 *
 */
public class SessionCreateConsumerDelegateRequest extends RequestSupport
{

   private JBossDestination dest;
   private String selector;
   private boolean noLocal;
   private String subName;
   private boolean connectionConsumer;

   public SessionCreateConsumerDelegateRequest()
   {      
   }
   
   public SessionCreateConsumerDelegateRequest(int objectId,
                                               byte version,
                                               JBossDestination destination,
                                               String selector, boolean noLocal,
                                               String subName, boolean connectionConsumer)
   {
      super(objectId, PacketSupport.REQ_SESSION_CREATECONSUMERDELEGATE, version);
      
      this.dest = destination;
      this.selector = selector;
      this.noLocal = noLocal;
      this.subName = subName;
      this.connectionConsumer = connectionConsumer;
   }

   public void read(DataInputStream is) throws Exception
   {
      super.read(is);
      
      dest = JBossDestination.readDestination(is);
      
      selector = readNullableString(is);
      
      noLocal = is.readBoolean();
      
      subName = readNullableString(is);
      
      connectionConsumer = is.readBoolean();
   }

   public ResponseSupport serverInvoke() throws Exception
   {
      SessionEndpoint endpoint = 
         (SessionEndpoint)Dispatcher.instance.getTarget(objectId);
      
      if (endpoint == null)
      {
         throw new IllegalStateException("Cannot find object in dispatcher with id " + objectId);
      }
      
      return new SessionCreateConsumerDelegateResponse((ClientConsumerDelegate)endpoint.createConsumerDelegate(dest, selector, noLocal, subName, connectionConsumer));
   }

   public void write(DataOutputStream os) throws Exception
   {
      super.write(os);
      
      JBossDestination.writeDestination(os, dest);
      
      writeNullableString(selector, os);
      
      os.writeBoolean(noLocal);
      
      writeNullableString(subName, os);
      
      os.writeBoolean(connectionConsumer);
      
      os.flush();
   }

}

